@@ -1,6 +1,7 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
- *  Copyright (C) 2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003-2004 Marco Pesenti Gritti
+ *  Copyright (C) 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,9 +17,15 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-favicon-cache.c,v 1.29 2004/01/25 13:50:37 chpe Exp $
+ *  $Id: ephy-favicon-cache.c,v 1.33 2004/02/29 17:59:32 chpe Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include "ephy-favicon-cache.h"
+
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <string.h>
 #include <time.h>
@@ -27,19 +34,18 @@
 #include "ephy-embed-persist.h"
 #include "ephy-embed-factory.h"
 #include "ephy-file-helpers.h"
-#include "ephy-favicon-cache.h"
 #include "ephy-node-common.h"
 #include "ephy-node.h"
 #include "ephy-debug.h"
 
 #define EPHY_FAVICON_CACHE_XML_ROOT    "ephy_favicons_cache"
-#define EPHY_FAVICON_CACHE_XML_VERSION "1.0"
+#define EPHY_FAVICON_CACHE_XML_VERSION "1.1"
 
 #define EPHY_FAVICON_CACHE_OBSOLETE_DAYS 30
 
 static void ephy_favicon_cache_class_init (EphyFaviconCacheClass *klass);
-static void ephy_favicon_cache_init (EphyFaviconCache *ma);
-static void ephy_favicon_cache_finalize (GObject *object);
+static void ephy_favicon_cache_init	  (EphyFaviconCache *cache);
+static void ephy_favicon_cache_finalize	  (GObject *object);
 
 #define EPHY_FAVICON_CACHE_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_FAVICON_CACHE, EphyFaviconCachePrivate))
 
@@ -59,16 +65,24 @@ enum
 	LAST_SIGNAL
 };
 
-static guint ephy_favicon_cache_signals[LAST_SIGNAL] = { 0 };
+enum
+{
+	EPHY_NODE_FAVICON_PROP_URL	 = 2,
+	EPHY_NODE_FAVICON_PROP_FILENAME	 = 3,
+	EPHY_NODE_FAVICON_PROP_LAST_USED = 4,
+	EPHY_NODE_FAVICON_PROP_STATE	 = 5
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
 
 static GObjectClass *parent_class = NULL;
 
 GType
 ephy_favicon_cache_get_type (void)
 {
-	static GType ephy_favicon_cache_type = 0;
+	static GType type = 0;
 
-	if (ephy_favicon_cache_type == 0)
+	if (type == 0)
 	{
 		static const GTypeInfo our_info =
 		{
@@ -83,12 +97,12 @@ ephy_favicon_cache_get_type (void)
 			(GInstanceInitFunc) ephy_favicon_cache_init
 		};
 
-		ephy_favicon_cache_type = g_type_register_static (G_TYPE_OBJECT,
-								    "EphyFaviconCache",
-								     &our_info, 0);
+		type = g_type_register_static (G_TYPE_OBJECT,
+					       "EphyFaviconCache",
+					       &our_info, 0);
 	}
 
-	return ephy_favicon_cache_type;
+	return type;
 }
 
 static void
@@ -100,7 +114,7 @@ ephy_favicon_cache_class_init (EphyFaviconCacheClass *klass)
 
 	object_class->finalize = ephy_favicon_cache_finalize;
 
-	ephy_favicon_cache_signals[CHANGED] =
+	signals[CHANGED] =
 		g_signal_new ("changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -111,7 +125,7 @@ ephy_favicon_cache_class_init (EphyFaviconCacheClass *klass)
 			      1,
 			      G_TYPE_STRING);
 
-	g_type_class_add_private (object_class, sizeof(EphyFaviconCachePrivate));
+	g_type_class_add_private (object_class, sizeof (EphyFaviconCachePrivate));
 }
 
 EphyFaviconCache *
@@ -206,7 +220,7 @@ ephy_favicon_cache_init (EphyFaviconCache *cache)
 						  NULL);
 
 	cache->priv->directory = g_build_filename (ephy_dot_dir (),
-						   "favicon_cache/",
+						   "favicon_cache",
 						   NULL);
 
 	if (g_file_test (cache->priv->directory, G_FILE_TEST_IS_DIR) == FALSE)
@@ -242,40 +256,36 @@ ephy_favicon_cache_init (EphyFaviconCache *cache)
 }
 
 static gboolean
-kill_download (gpointer key,
-	       gpointer value,
-	       gpointer data)
+kill_download (char *key,
+	       EphyEmbedPersist *persist,
+	       EphyFaviconCache *cache)
 {
-	EphyEmbedPersist *persist = EPHY_EMBED_PERSIST (value);
-	EphyFaviconCache *cache = EPHY_FAVICON_CACHE (data);
 	EphyNode *icon;
 
-	ephy_embed_persist_cancel (persist);
-	g_object_unref (persist);
+	/* disconnect "completed" and "cancelled" callbacks */
+	g_signal_handlers_disconnect_matched
+		(persist,G_SIGNAL_MATCH_DATA , 0, 0, NULL, NULL, cache);
 
-	icon = g_hash_table_lookup (cache->priv->icons_hash, (char *)key);
+	/* now cancel the download */
+	ephy_embed_persist_cancel (persist);
+
+	icon = g_hash_table_lookup (cache->priv->icons_hash, key);
+	g_return_val_if_fail (EPHY_IS_NODE (icon), TRUE);
 
 	ephy_node_unref (icon);
 
 	return TRUE;
 }
 
-
-static void
-cleanup_downloads_hash (EphyFaviconCache *cache)
-{
-	g_hash_table_foreach_remove (cache->priv->downloads_hash,
-				     kill_download, cache);
-}
-
 static void
 ephy_favicon_cache_finalize (GObject *object)
 {
 	EphyFaviconCache *cache = EPHY_FAVICON_CACHE (object);
 
-	LOG ("Finalize favicon cache")
+	LOG ("EphyFaviconCache finalising")
 
-	cleanup_downloads_hash (cache);
+	g_hash_table_foreach_remove (cache->priv->downloads_hash,
+				     (GHRFunc) kill_download, cache);
 	remove_obsolete_icons (cache);
 
 	ephy_node_db_write_to_xml_safe
@@ -298,15 +308,16 @@ ephy_favicon_cache_finalize (GObject *object)
 static char *
 favicon_name_build (const char *url)
 {
-	char *res;
-	char *slashpos;
+	char *result;
 
-	res = g_strdup (url);
+	result = g_filename_from_utf8 (url, -1, NULL, NULL, NULL);
 
-	while ((slashpos = strstr (res, "/")) != NULL)
-		*slashpos = '_';
+	if (result == NULL)
+	{
+		return NULL;
+	}
 
-	return res;
+	return g_strdelimit (result, "/", '_');
 }
 
 static void
@@ -318,9 +329,33 @@ favicon_download_completed_cb (EphyEmbedPersist *persist,
 	url = ephy_embed_persist_get_source (persist);
 	g_return_if_fail (url != NULL);
 
+	LOG ("Favicon cache download completed for %s", url)
+
 	g_hash_table_remove (cache->priv->downloads_hash, url);
 
-	g_signal_emit (G_OBJECT (cache), ephy_favicon_cache_signals[CHANGED], 0, url);
+	g_signal_emit (G_OBJECT (cache), signals[CHANGED], 0, url);
+
+	g_object_unref (persist);
+}
+
+static void
+favicon_download_cancelled_cb (EphyEmbedPersist *persist,
+			       EphyFaviconCache *cache)
+{
+	const char *url, *dest;
+
+	url = ephy_embed_persist_get_source (persist);
+	g_return_if_fail (url != NULL);
+
+	LOG ("Favicon cache download cancelled %s", url)
+
+	g_hash_table_remove (cache->priv->downloads_hash, url);
+
+	/* remove a partially downloaded file */
+	dest = ephy_embed_persist_get_dest (persist);
+	gnome_vfs_unlink (dest);
+
+	/* FIXME: re-schedule to try again after n days? */
 
 	g_object_unref (persist);
 }
@@ -346,15 +381,15 @@ ephy_favicon_cache_download (EphyFaviconCache *cache,
 
 	ephy_embed_persist_set_dest (persist, dest);
 	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_NO_VIEW);
-	ephy_embed_persist_set_max_size (persist, 100);
+	ephy_embed_persist_set_max_size (persist, 100 * 1024);
 	ephy_embed_persist_set_source (persist, favicon_url);
 
 	g_free (dest);
 
-	g_signal_connect (G_OBJECT (persist),
-			  "completed",
-			  G_CALLBACK (favicon_download_completed_cb),
-			  cache);
+	g_signal_connect (G_OBJECT (persist), "completed",
+			  G_CALLBACK (favicon_download_completed_cb), cache);
+	g_signal_connect (G_OBJECT (persist), "cancelled",
+			  G_CALLBACK (favicon_download_cancelled_cb), cache);
 
 	g_hash_table_insert (cache->priv->downloads_hash,
 			     g_strdup (favicon_url), persist);
@@ -383,6 +418,10 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 		char *filename;
 
 		filename = favicon_name_build (url);
+		if (filename == NULL)
+		{
+			return NULL;
+		}
 
 		icon = ephy_node_new (cache->priv->db);
 		g_value_init (&value, G_TYPE_STRING);
@@ -425,9 +464,15 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 
 	pixbuf = gdk_pixbuf_new_from_file (pix_file, NULL);
 
-	if (pixbuf &&
-	    (gdk_pixbuf_get_width (pixbuf) > 16 ||
-	     gdk_pixbuf_get_height (pixbuf) > 16))
+	g_free (pix_file);
+
+	if (pixbuf == NULL)
+	{
+		return NULL;
+	}
+
+	if (gdk_pixbuf_get_width (pixbuf) > 16 ||
+	    gdk_pixbuf_get_height (pixbuf) > 16)
 	{
 		GdkPixbuf *scaled = gdk_pixbuf_scale_simple (pixbuf, 16, 16,
 							     GDK_INTERP_NEAREST);
@@ -435,7 +480,5 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 		pixbuf = scaled;
 	}
 
-	g_free (pix_file);
-
 	return pixbuf;
 }
