@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.cpp,v 1.22.4.8 2004/07/31 15:48:28 chpe Exp $
+ *  $Id: EphyBrowser.cpp,v 1.22.4.11 2004/08/12 10:27:14 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -77,7 +77,11 @@
 #include "nsIDOMHTMLInputElement.h"
 #include "nsIDOMHTMLTextAreaElement.h"
 #include "nsIDeviceContext.h"
+#if MOZILLA_SNAPSHOT >= 20
+#include "nsPresContext.h"
+#else
 #include "nsIPresContext.h"
+#endif
 #include "nsIAtom.h"
 #include "nsIDocumentCharsetInfo.h"
 #include "nsPromiseFlatString.h"
@@ -86,7 +90,7 @@
 #include "MozillaPrivate.h"
 #include "print-dialog.h"
 
-#if MOZILLA_SNAPSHOT >= 17
+#if (MOZILLA_SNAPSHOT == 17) || (MOZILLA_SNAPSHOT >= 19)
 #include "nsIDOMWindow2.h"
 #endif
 
@@ -124,8 +128,13 @@ EphyFaviconEventListener::HandleFaviconLink (nsIDOMNode *node)
 	result = linkElement->GetAttribute (attr_rel, value);
 	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 
+#if MOZILLA_SNAPSHOT >= 20
+	if (value.LowerCaseEqualsASCII("shortcut icon") ||
+	    value.LowerCaseEqualsASCII("icon"))
+#else
 	if (value.EqualsIgnoreCase("SHORTCUT ICON") ||
 	    value.EqualsIgnoreCase("ICON"))
+#endif
 	{
 		NS_NAMED_LITERAL_STRING(attr_href, "href");
 		nsAutoString value;
@@ -249,7 +258,7 @@ EphyBrowser::GetListener (void)
   	nsCOMPtr<nsIDOMWindow> domWindowExternal;
   	mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindowExternal));
 
-#if MOZILLA_SNAPSHOT >= 17
+#if (MOZILLA_SNAPSHOT == 17) || (MOZILLA_SNAPSHOT >= 19)
 	nsCOMPtr<nsIDOMWindow2> domWindow (do_QueryInterface (domWindowExternal));
 	NS_ENSURE_TRUE (domWindow, NS_ERROR_FAILURE);
 
@@ -513,7 +522,11 @@ nsresult EphyBrowser::SetZoom (float aZoom, PRBool reflow)
 
 nsresult EphyBrowser::SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell)
 {
+#if MOZILLA_SNAPSHOT >= 20
+	nsCOMPtr<nsPresContext> PresContext;
+#else
 	nsCOMPtr<nsIPresContext> PresContext;
+#endif
 	DocShell->GetPresContext (getter_AddRefs(PresContext));
 	NS_ENSURE_TRUE (PresContext, NS_ERROR_FAILURE);
 
@@ -992,7 +1005,11 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 			nsAutoString type;
 			inputElement->GetType(type);
 
+#if MOZILLA_SNAPSHOT >= 20
+			if (type.LowerCaseEqualsASCII("text"))
+#else
 			if (type.EqualsIgnoreCase("text"))
+#endif
 			{
 				nsAutoString default_text, user_text;
 				PRInt32 max_length;
