@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GlobalHistory.cpp,v 1.16.4.2 2004/06/10 16:41:45 chpe Exp $
+ *  $Id: GlobalHistory.cpp,v 1.16.4.7 2004/08/13 14:37:38 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,13 +51,9 @@ MozGlobalHistory::~MozGlobalHistory ()
 NS_IMETHODIMP MozGlobalHistory::AddURI(nsIURI *aURI, PRBool aRedirect, PRBool aToplevel)
 {
 	nsresult rv;
-	NS_ENSURE_ARG_POINTER(aURI);
+	NS_ENSURE_ARG (aURI);
 
-	PRBool isJavascript;
-	rv = aURI->SchemeIs("javascript", &isJavascript);
-	NS_ENSURE_SUCCESS(rv, rv);
-
-	if (isJavascript || aRedirect || !aToplevel)
+	if (aRedirect || !aToplevel)
 	{
 		return NS_OK;
 	}
@@ -78,15 +74,16 @@ NS_IMETHODIMP MozGlobalHistory::AddURI(nsIURI *aURI, PRBool aRedirect, PRBool aT
 
 	if (!isHTTP && !isHTTPS)
 	{
-		PRBool isAbout, isViewSource, isChrome, isData;
+		PRBool isJavascript, isAbout, isViewSource, isChrome, isData;
 
-		rv = aURI->SchemeIs("about", &isAbout);
+		rv = aURI->SchemeIs("javascript", &isJavascript);
+		rv |= aURI->SchemeIs("about", &isAbout);
 		rv |= aURI->SchemeIs("view-source", &isViewSource);
 		rv |= aURI->SchemeIs("chrome", &isChrome);
 		rv |= aURI->SchemeIs("data", &isData);
 		NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
 
-		if (isAbout || isViewSource || isChrome || isData)
+		if (isJavascript || isAbout || isViewSource || isChrome || isData)
 		{
 		      return NS_OK;
 		}
@@ -104,7 +101,7 @@ NS_IMETHODIMP MozGlobalHistory::AddURI(nsIURI *aURI, PRBool aRedirect, PRBool aT
 /* boolean isVisited (in nsIURI aURI); */
 NS_IMETHODIMP MozGlobalHistory::IsVisited(nsIURI *aURI, PRBool *_retval)
 {
-	NS_ENSURE_ARG_POINTER(aURI);
+	NS_ENSURE_ARG (aURI);
 
 	nsCAutoString spec;
 	aURI->GetSpec(spec);
@@ -117,12 +114,14 @@ NS_IMETHODIMP MozGlobalHistory::IsVisited(nsIURI *aURI, PRBool *_retval)
 /* void setPageTitle (in nsIURI aURI, in AString aTitle); */
 NS_IMETHODIMP MozGlobalHistory::SetPageTitle(nsIURI *aURI, const nsAString & aTitle)
 {
-	NS_ENSURE_ARG_POINTER(aURI);
+	NS_ENSURE_ARG (aURI);
 
 	const nsACString &title = NS_ConvertUTF16toUTF8(aTitle);
 
+	nsresult rv;
 	nsCAutoString spec;
-	aURI->GetSpec(spec);
+	rv = aURI->GetSpec(spec);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 	
 	ephy_history_set_page_title (mGlobalHistory, spec.get(),
 				     PromiseFlatCString(title).get());
@@ -131,7 +130,7 @@ NS_IMETHODIMP MozGlobalHistory::SetPageTitle(nsIURI *aURI, const nsAString & aTi
 }
 
 /* void hidePage (in nsIURI url); */
-NS_IMETHODIMP MozGlobalHistory::HidePage(nsIURI *url)
+NS_IMETHODIMP MozGlobalHistory::HidePage(nsIURI *aURI)
 {
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -141,6 +140,8 @@ NS_IMETHODIMP MozGlobalHistory::HidePage(nsIURI *url)
 /* void addPage (in string aURL); */
 NS_IMETHODIMP MozGlobalHistory::AddPage (const char *aURL)
 {
+	NS_ENSURE_ARG (aURL);
+
 	ephy_history_add_page (mGlobalHistory, aURL);
 	
 	return NS_OK;
@@ -149,6 +150,8 @@ NS_IMETHODIMP MozGlobalHistory::AddPage (const char *aURL)
 /* boolean isVisited (in string aURL); */
 NS_IMETHODIMP MozGlobalHistory::IsVisited (const char *aURL, PRBool *_retval)
 {
+	NS_ENSURE_ARG (aURL);
+
 	*_retval = ephy_history_is_page_visited (mGlobalHistory, aURL);
 	
 	return NS_OK;
@@ -158,6 +161,7 @@ NS_IMETHODIMP MozGlobalHistory::IsVisited (const char *aURL, PRBool *_retval)
 NS_IMETHODIMP MozGlobalHistory::SetPageTitle (const char *aURL, 
 					      const PRUnichar *aTitle)
 {
+	NS_ENSURE_ARG (aURL);
 	const nsACString &title = NS_ConvertUCS2toUTF8 (aTitle);
 	
 	ephy_history_set_page_title (mGlobalHistory, aURL, PromiseFlatCString(title).get());
@@ -166,13 +170,38 @@ NS_IMETHODIMP MozGlobalHistory::SetPageTitle (const char *aURL,
 	return NS_OK;
 }
 
-NS_IMETHODIMP MozGlobalHistory::HidePage(const char *url)
+NS_IMETHODIMP MozGlobalHistory::HidePage(const char *aURL)
 {
-	return NS_ERROR_NOT_IMPLEMENTED;
+        NS_ENSURE_ARG (aURL);
 
+        EphyNode *page;
+        page = ephy_history_get_page (mGlobalHistory, aURL);
+
+        if (page)
+        {
+                ephy_node_unref (page);
+        }
+
+        return NS_OK;
 }
 #endif /* MOZILLA_SNAPSHOT > 13 */
 
+#if MOZILLA_SNAPSHOT >= 20
+/* void removePage (in nsIURI aURL); */
+NS_IMETHODIMP MozGlobalHistory::RemovePage(nsIURI *aURI)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* void removePagesFromHost (in AUTF8String aHost, in boolean aEntireDomain); */
+NS_IMETHODIMP MozGlobalHistory::RemovePagesFromHost(const nsACString &aHost, 
+						    PRBool aEntireDomain)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+#else
+
 /* void removePage (in string aURL); */
 NS_IMETHODIMP MozGlobalHistory::RemovePage(const char *aURL)
 {
@@ -185,7 +214,7 @@ NS_IMETHODIMP MozGlobalHistory::RemovePagesFromHost(const char *aHost,
 {
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
-
+#endif
 /* void removeAllPages (); */
 NS_IMETHODIMP MozGlobalHistory::RemoveAllPages()
 {
@@ -219,8 +248,16 @@ NS_IMETHODIMP MozGlobalHistory::GetCount(PRUint32 *aCount)
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+#if MOZILLA_SNAPSHOT >= 20
+/* void markPageAsTyped (in nsIURI url); */
+NS_IMETHODIMP MozGlobalHistory::MarkPageAsTyped(nsIURI *aURI)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+#else
 /* void markPageAsTyped (in string url); */
 NS_IMETHODIMP MozGlobalHistory::MarkPageAsTyped(const char *url)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
+#endif
