@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2001 Philip Langdale
+ *  Copyright (C) 2001, 2004 Philip Langdale
+ *  Copyright (C) 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,39 +15,56 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: GlobalHistory.h,v 1.5 2004/02/12 16:59:25 chpe Exp $
  */
 
-#ifndef __GlobalHistory_h
-#define __GlobalHistory_h
+#ifndef EPHY_GLOBAL_HISTORY_H
+#define EPHY_GLOBAL_HISTORY_H
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "nsError.h"
-#include "nsIGlobalHistory.h"
 #include "nsIBrowserHistory.h"
 #include "ephy-history.h"
 
-#define EPHY_GLOBALHISTORY_CLASSNAME \
- "Epiphany's Global History Implementation"
+#if MOZILLA_SNAPSHOT > 13
+#include <nsIGlobalHistory2.h>
+#else
+#include <nsIGlobalHistory.h>
+#endif
 
-#define EPHY_GLOBALHISTORY_CID \
- { 0xbe0c42c1, 0x39d4, 0x4271, { 0xb7, 0x9e, 0xf7, 0xaa, 0x49, 0xeb, 0x6a, 0x15}}
+#define EPHY_GLOBALHISTORY_CLASSNAME	"Epiphany Global History Implementation"
 
-/**
- * class GlobalHistory:
- *
- */
-class MozGlobalHistory: public nsIGlobalHistory,
-	public nsIBrowserHistory
+#define EPHY_GLOBALHISTORY_CID					\
+{	0xbe0c42c1,						\
+	0x39d4,							\
+	0x4271,							\
+	{ 0xb7, 0x9e, 0xf7, 0xaa, 0x49, 0xeb, 0x6a, 0x15}	\
+}
+
+class MozGlobalHistory:
+#if MOZILLA_SNAPSHOT <= 13
+			public nsIGlobalHistory,
+#endif
+			public nsIBrowserHistory
 {
 	public:
 		MozGlobalHistory ();
 		virtual ~MozGlobalHistory();
 
 		NS_DECL_ISUPPORTS
+#if MOZILLA_SNAPSHOT > 13
+		NS_DECL_NSIGLOBALHISTORY2
+#else
 		NS_DECL_NSIGLOBALHISTORY
+#endif
 		NS_DECL_NSIBROWSERHISTORY
 
 	private:
 		EphyHistory *mGlobalHistory;
 };
 
-#endif
+#endif /* EPHY_GLOBAL_HISTORY_H */
