@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: print-dialog.c,v 1.23 2004/01/23 18:07:43 chpe Exp $
+ *  $Id: print-dialog.c,v 1.23.4.4 2004/07/17 14:36:02 chpe Exp $
  */
 
 #include "print-dialog.h"
@@ -131,8 +131,6 @@ void ephy_print_dialog_response_cb		(GtkWidget *widget,
 						 EphyDialog *dialog);
 void ephy_print_dialog_browse_button_cb		(GtkWidget *widget,
 						 EphyDialog *dialog);
-void ephy_print_setup_dialog_close_button_cb	(GtkWidget *widget,
-						 EphyDialog *dialog);
 void ephy_print_setup_dialog_help_button_cb	(GtkWidget *widget,
 						 EphyDialog *dialog);
 
@@ -164,11 +162,7 @@ ephy_print_get_print_info (void)
 	filename = eel_gconf_get_string (print_props[FILE_PROP].pref);
 	if (filename != NULL)
 	{
-		char *expanded;
-
-		expanded = gnome_vfs_expand_initial_tilde (filename);
-		info->file = g_filename_to_utf8 (expanded, -1, NULL, NULL, NULL);
-		g_free (expanded);
+		info->file = gnome_vfs_expand_initial_tilde (filename);
 	}
 	else
 	{
@@ -204,7 +198,6 @@ ephy_print_get_print_info (void)
 	info->footer_center_string = g_strdup("");
 
 	info->frame_type = 0;
-	info->preview = FALSE;
 
 	return info;
 }
@@ -214,39 +207,14 @@ ephy_print_dialog_response_cb (GtkWidget *widget,
 			       int response,
 			       EphyDialog *dialog)
 {
-	EphyEmbed *embed;
-	EmbedPrintInfo *info;
-	gboolean only_collect_info;
-
-	only_collect_info = GPOINTER_TO_INT
-		(g_object_get_data (G_OBJECT (dialog), "only-collect-info"));
-
-	if (only_collect_info)
-	{
-		return;
-	}
-
 	switch (response)
 	{
-		case GTK_RESPONSE_OK:
-			info = ephy_print_get_print_info ();
-	
-			embed = ephy_embed_dialog_get_embed (EPHY_EMBED_DIALOG (dialog));
-			g_return_if_fail (EPHY_IS_EMBED (embed));
-	
-			ephy_embed_print (embed, info);
-	
-			ephy_print_info_free (info);
-
-			break;
 		case GTK_RESPONSE_HELP:
 			ephy_gui_help (GTK_WINDOW (widget), "epiphany", "to-print-page");
 			return;
 		default:
 			break;
 	}
-
-	g_object_unref (dialog);
 }
 
 static void
@@ -254,7 +222,7 @@ print_filechooser_response_cb (GtkDialog *fc,
 			       int response,
 			       EphyDialog *dialog)
 {
-	if (response == EPHY_RESPONSE_SAVE)
+	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		char *filename;
 
@@ -295,27 +263,20 @@ ephy_print_dialog_browse_button_cb (GtkWidget *widget,
 			  G_CALLBACK (print_filechooser_response_cb),
 			  dialog);
 
+	gtk_window_set_modal (GTK_WINDOW (fc), TRUE);
 	gtk_widget_show (GTK_WIDGET (fc));
 }
 
-void
-ephy_print_setup_dialog_close_button_cb (GtkWidget *widget,
-					 EphyDialog *dialog)
-{
-	g_object_unref (dialog);
-}
-
 void
 ephy_print_setup_dialog_help_button_cb (GtkWidget *widget,
-					 EphyDialog *dialog)
+					EphyDialog *dialog)
 {
-	ephy_gui_help (GTK_WINDOW (dialog), "epiphany", "using-print-setup");
+	ephy_gui_help (GTK_WINDOW (widget), "epiphany", "using-print-setup");
 }
 
 EphyDialog *
 ephy_print_dialog_new (GtkWidget *parent,
-		       EphyEmbed *embed,
-		       gboolean only_collect_info)
+		       EphyEmbed *embed)
 {
 	EphyDialog *dialog;
 	GtkWidget *window, *button;
@@ -347,9 +308,6 @@ ephy_print_dialog_new (GtkWidget *parent,
 	button = ephy_dialog_get_control (dialog, print_props[BROWSE_PROP].id);
 	gtk_widget_set_sensitive (button, eel_gconf_key_is_writable (CONF_PRINT_FILE));
 
-	g_object_set_data (G_OBJECT (dialog), "only-collect-info",
-			   GINT_TO_POINTER (only_collect_info));
-
 	return dialog;
 }
 
