@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.c,v 1.84 2004/02/24 17:43:48 chpe Exp $
+ *  $Id: ephy-shell.c,v 1.84.4.2 2004/08/06 14:41:30 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,7 +76,6 @@ struct EphyShellPrivate
 	GtkWidget *history_window;
 	GObject *pdm_dialog;
 	GObject *prefs_dialog;
-	GObject *print_setup_dialog;
 	GList *del_on_exit;
 	guint server_timeout;
 };
@@ -192,7 +191,6 @@ ephy_shell_init (EphyShell *shell)
 	shell->priv->bme = NULL;
 	shell->priv->history_window = NULL;
 	shell->priv->pdm_dialog = NULL;
-	shell->priv->print_setup_dialog = NULL;
 	shell->priv->toolbars_model = NULL;
 	shell->priv->fs_toolbars_model = NULL;
 	shell->priv->extensions_manager = NULL;
@@ -483,12 +481,6 @@ ephy_shell_finalize (GObject *object)
 		g_object_unref (shell->priv->prefs_dialog);
 	}
 
-	LOG ("Unref print setup dialog")
-	if (shell->priv->print_setup_dialog)
-	{
-		g_object_unref (shell->priv->print_setup_dialog);
-	}
-
 	LOG ("Unref bookmarks")
 	if (shell->priv->bookmarks)
 	{
@@ -588,11 +580,11 @@ ephy_shell_new_tab (EphyShell *shell,
 		    flags & EPHY_NEW_TAB_APPEND_GROUPED)) &&
 		  !(flags & EPHY_NEW_TAB_APPEND_LAST);
 
-	if ((flags & EPHY_NEW_TAB_APPEND_AFTER) && previous_embed != NULL)
+	if ((flags & EPHY_NEW_TAB_APPEND_AFTER) && previous_tab != NULL)
 	{
 		nb = ephy_window_get_notebook (window);
 		position = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-						  GTK_WIDGET (previous_embed)) + 1;
+						  GTK_WIDGET (previous_tab)) + 1;
 	}
 	else
 	{
@@ -814,17 +806,3 @@ ephy_shell_get_prefs_dialog (EphyShell *shell)
 
 	return shell->priv->prefs_dialog;
 }
-
-GObject *
-ephy_shell_get_print_setup_dialog (EphyShell *shell)
-{
-	if (shell->priv->print_setup_dialog == NULL)
-	{
-		shell->priv->print_setup_dialog = G_OBJECT (ephy_print_setup_dialog_new ());
-
-		g_object_add_weak_pointer (shell->priv->print_setup_dialog,
-					   (gpointer *) &shell->priv->print_setup_dialog);
-	}
-
-	return shell->priv->print_setup_dialog;
-}
