@@ -1,6 +1,7 @@
 /*
  *  Copyright (C) 2000-2003 Marco Pesenti Gritti
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright (C) 2004 Crispin Flowerday
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.89 2004/02/11 16:10:06 marco Exp $
+ *  $Id: ephy-tab.c,v 1.95.4.2 2004/06/08 23:46:33 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -59,9 +60,10 @@
 
 #define EPHY_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TAB, EphyTabPrivate))
 
+#define CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME  "/apps/epiphany/lockdown/disable_javascript_chrome"
+
 struct EphyTabPrivate
 {
-	EphyEmbed *embed;
 	EphyWindow *window;
 	char *status_message;
 	char *link_message;
@@ -116,10 +118,12 @@ static void	ephy_tab_set_link_message	(EphyTab *tab,
 						 char *message);
 static void	ephy_tab_set_load_percent	(EphyTab *tab,
 						 int percent);
-static void	ephy_tab_update_navigation_flags(EphyTab *tab);
+static void	ephy_tab_update_navigation_flags(EphyTab *tab,
+						 EphyEmbed *embed);
 static void	ephy_tab_set_security_level	(EphyTab *tab,
 						 EmbedSecurityLevel level);
 static void	ephy_tab_set_title		(EphyTab *tab,
+						 EphyEmbed *embed,
 						 const char *new_title);
 static void	ephy_tab_set_zoom		(EphyTab *tab,
 						 float zoom);
@@ -146,7 +150,7 @@ ephy_tab_get_type (void)
                         (GInstanceInitFunc) ephy_tab_init
                 };
 
-                ephy_tab_type = g_type_register_static (G_TYPE_OBJECT,
+                ephy_tab_type = g_type_register_static (GTK_TYPE_BIN,
 							"EphyTab",
 							&our_info, 0);
         }
@@ -232,6 +236,40 @@ ephy_tab_get_property (GObject *object,
 	}
 }
 
+static void
+ephy_tab_size_allocate (GtkWidget *widget,
+			GtkAllocation *allocation)
+{
+	GtkWidget *child;
+
+	widget->allocation = *allocation;
+
+	child = GTK_BIN (widget)->child;
+
+	if (child && GTK_WIDGET_VISIBLE (child))
+	{
+		gtk_widget_size_allocate (child, allocation);
+	}
+}
+
+static void
+ephy_tab_parent_set (GtkWidget *widget,
+		     GtkWidget *previous_parent)
+{
+	if (widget->parent)
+	{
+		GtkWidget *toplevel;
+		toplevel = gtk_widget_get_toplevel (widget);
+
+		ephy_tab_set_window (EPHY_TAB (widget), EPHY_WINDOW (toplevel));
+	}
+
+	if (GTK_WIDGET_CLASS (parent_class)->parent_set)
+	{
+		GTK_WIDGET_CLASS (parent_class)->parent_set (widget, previous_parent);
+	}
+}
+
 static void
 ephy_tab_action_activate_cb (GtkAction *action, EphyTab *tab)
 {
@@ -248,6 +286,7 @@ static void
 ephy_tab_class_init (EphyTabClass *class)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (class);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(class);
 
         parent_class = g_type_class_peek_parent (class);
 
@@ -255,6 +294,9 @@ ephy_tab_class_init (EphyTabClass *class)
 	object_class->get_property = ephy_tab_get_property;
 	object_class->set_property = ephy_tab_set_property;
 
+	widget_class->size_allocate = ephy_tab_size_allocate;
+	widget_class->parent_set = ephy_tab_parent_set;
+	
 	g_object_class_install_property (object_class,
 					 PROP_ADDRESS,
 					 g_param_spec_string ("address",
@@ -310,7 +352,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							    G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
-					 PROP_LOAD_STATUS,
+					 PROP_SECURITY,
 					 g_param_spec_int ("security-level",
 							   "Security Level",
 							   "The tab's security level",
@@ -328,7 +370,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
-					 PROP_LOAD_STATUS,
+					 PROP_VISIBLE,
 					 g_param_spec_boolean ("visible",
 							       "Visibility",
 							       "The tab's visibility",
@@ -356,32 +398,14 @@ ephy_tab_class_init (EphyTabClass *class)
 	g_type_class_add_private (object_class, sizeof(EphyTabPrivate));
 }
 
-static void
-ephy_tab_parent_set_cb (GtkWidget *widget, GtkWidget *previous_parent,
-			EphyTab *tab)
-{
-	GtkWidget *toplevel;
-
-	if (widget->parent == NULL) return;
-
-	toplevel = gtk_widget_get_toplevel (widget);
-	ephy_tab_set_window (tab, EPHY_WINDOW (toplevel));
-}
-static void
-ephy_tab_embed_destroy_cb (GtkWidget *widget, EphyTab *tab)
-{
-	LOG ("GtkMozEmbed destroy signal on EphyTab")
-	g_object_unref (tab);
-}
-
 static void
 ephy_tab_finalize (GObject *object)
 {
         EphyTab *tab = EPHY_TAB (object);
 
-	g_idle_remove_by_data (tab->priv->embed);
+	g_idle_remove_by_data (tab);
 
-	if (tab->priv->action)
+	if (tab->priv->action != NULL)
 	{
 		g_object_unref (tab->priv->action);
 	}
@@ -400,25 +424,17 @@ ephy_tab_finalize (GObject *object)
 static gboolean
 address_has_web_scheme (const char *address)
 {
-	GnomeVFSURI *uri;
-	const char *scheme;
-	gboolean has_web_scheme = FALSE;
+	gboolean has_web_scheme;
 
-	uri = gnome_vfs_uri_new (address);
-	if (uri != NULL)
-	{
-		scheme = gnome_vfs_uri_get_scheme (uri);
+	if (address == NULL) return FALSE;
 
-		has_web_scheme = (strcmp (scheme, "http") == 0 ||
-				  strcmp (scheme, "https") == 0 ||
-				  strcmp (scheme, "ftp") == 0 ||
-				  strcmp (scheme, "file") == 0 ||
-				  strcmp (scheme, "data") == 0 ||
-				  strcmp (scheme, "about") == 0 ||
-				  strcmp (scheme, "gopher") == 0);
-
-		gnome_vfs_uri_unref (uri);
-	}
+	has_web_scheme = (g_str_has_prefix (address, "http:") ||
+			  g_str_has_prefix (address, "https:") ||
+			  g_str_has_prefix (address, "ftp:") ||
+			  g_str_has_prefix (address, "file:") ||
+			  g_str_has_prefix (address, "data:") ||
+			  g_str_has_prefix (address, "about:") ||
+			  g_str_has_prefix (address, "gopher:"));
 
 	return has_web_scheme;
 }
@@ -428,11 +444,7 @@ address_has_web_scheme (const char *address)
 EphyTab *
 ephy_tab_new (void)
 {
-	EphyTab *tab;
-
-	tab = EPHY_TAB (g_object_new (EPHY_TYPE_TAB, NULL));
-
-	return tab;
+	return EPHY_TAB (g_object_new (EPHY_TYPE_TAB, NULL));
 }
 
 static void
@@ -477,19 +489,20 @@ ephy_tab_get_embed (EphyTab *tab)
 {
 	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
-	return tab->priv->embed;
+	return EPHY_EMBED (gtk_bin_get_child (GTK_BIN (tab)));
 }
 
 EphyTab *
 ephy_tab_for_embed (EphyEmbed *embed)
 {
-	EphyTab *tab;
+	GtkWidget *parent;
 
 	g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
 
-	tab = EPHY_TAB (g_object_get_data (G_OBJECT (embed), "ephy-tab"));
+	parent = GTK_WIDGET (embed)->parent;
+	g_return_val_if_fail (parent != NULL, NULL);
 
-	return tab;
+	return EPHY_TAB (parent);
 }
 
 void
@@ -612,16 +625,29 @@ ephy_tab_link_message_cb (EphyEmbed *embed,
 static void
 ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 {
+	const char *uv_address;
+
 	LOG ("ephy_tab_address_cb tab %p address %s", tab, address)
 
+	/* Do not expose about:blank to the user, an empty address
+	   bar will do better */
+	if (address && strcmp (address, "about:blank") == 0)
+	{
+		uv_address = "";
+	}
+	else
+	{
+		uv_address = address;
+	}
+
 	if (tab->priv->address_expire == TAB_ADDRESS_EXPIRE_NOW)
 	{
-		ephy_tab_set_location (tab, address, TAB_ADDRESS_EXPIRE_NOW);
+		ephy_tab_set_location (tab, uv_address, TAB_ADDRESS_EXPIRE_NOW);
 	}
 
 	ephy_tab_set_link_message (tab, NULL);
 	ephy_tab_set_icon_address (tab, NULL);
-	ephy_tab_update_navigation_flags (tab);
+	ephy_tab_update_navigation_flags (tab, embed);
 
 	/* restore zoom level */
 	if (address_has_web_scheme (address))
@@ -700,7 +726,7 @@ ephy_tab_title_cb (EphyEmbed *embed, EphyTab *tab)
 
 	title = ephy_embed_get_title (embed);
 
-	ephy_tab_set_title (tab, title);
+	ephy_tab_set_title (tab, embed, title);
 
 	g_free (title);
 }
@@ -831,9 +857,9 @@ build_progress_from_requests (EphyTab *tab, EmbedState state)
 }
 
 static void
-ensure_page_info (EphyTab *tab, const char *address)
+ensure_page_info (EphyTab *tab, EphyEmbed *embed, const char *address)
 {
-	if (tab->priv->address == NULL &&
+	if ((tab->priv->address == NULL || *tab->priv->address == '\0') &&
 	    tab->priv->address_expire == TAB_ADDRESS_EXPIRE_NOW)
         {
 		ephy_tab_set_location (tab, address, TAB_ADDRESS_EXPIRE_NOW);
@@ -841,7 +867,7 @@ ensure_page_info (EphyTab *tab, const char *address)
 
 	if (tab->priv->title == NULL)
 	{
-		ephy_tab_set_title (tab, NULL);
+		ephy_tab_set_title (tab, embed, NULL);
 	}
 }
 
@@ -857,17 +883,17 @@ ephy_tab_net_state_cb (EphyEmbed *embed, const char *uri,
 		{
 			tab->priv->total_requests = 0;
 			tab->priv->cur_requests = 0;
-			ensure_page_info (tab, uri);
+			ensure_page_info (tab, embed, uri);
 
 			ephy_tab_set_load_percent (tab, 0);
 			ephy_tab_set_load_status (tab, TRUE);
-			ephy_tab_update_navigation_flags (tab);
+			ephy_tab_update_navigation_flags (tab, embed);
 		}
 		else if (state & EMBED_STATE_STOP)
 		{
 			ephy_tab_set_load_percent (tab, 0);
 			ephy_tab_set_load_status (tab, FALSE);
-			ephy_tab_update_navigation_flags (tab);
+			ephy_tab_update_navigation_flags (tab, embed);
 			tab->priv->address_expire = TAB_ADDRESS_EXPIRE_NOW;
 		}
 	}
@@ -877,39 +903,47 @@ ephy_tab_net_state_cb (EphyEmbed *embed, const char *uri,
 
 static void
 ephy_tab_new_window_cb (EphyEmbed *embed, EphyEmbed **new_embed,
-			EmbedChromeMask chromemask, EphyTab *tab)
+			EphyEmbedChrome chromemask, EphyTab *tab)
 {
 	EphyTab *new_tab;
 	EphyWindow *window;
 
-	window = ephy_window_new ();
-	ephy_window_request_chrome (window, chromemask);
+	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME))
+	{
+		window = ephy_window_new ();
+	}
+	else
+	{
+		window = ephy_window_new_with_chrome (chromemask);
+	}
 
 	new_tab = ephy_tab_new ();
+	gtk_widget_show (GTK_WIDGET (new_tab));
+
         ephy_window_add_tab (window, new_tab, EPHY_NOTEBOOK_INSERT_GROUPED, FALSE);
 
 	*new_embed = ephy_tab_get_embed (new_tab);
 }
 
 static gboolean
-let_me_resize_hack (gpointer data)
+let_me_resize_hack (GtkWidget *tab)
 {
-	gtk_widget_set_size_request (GTK_WIDGET(data),
-				     -1, -1);
+	gtk_widget_set_size_request (tab, -1, -1);
+
 	return FALSE;
 }
 
 static void
 ephy_tab_visibility_cb (EphyEmbed *embed, gboolean visibility,
-			  EphyTab *tab)
+			EphyTab *tab)
 {
 	if (visibility)
 	{
-		gtk_widget_show (GTK_WIDGET(embed));
+		gtk_widget_show (GTK_WIDGET (tab));
 	}
 	else
 	{
-		gtk_widget_hide (GTK_WIDGET(embed));
+		gtk_widget_hide (GTK_WIDGET (tab));
 	}
 
 	ephy_tab_set_visibility (tab, visibility);
@@ -918,10 +952,15 @@ ephy_tab_visibility_cb (EphyEmbed *embed, gboolean visibility,
 static void
 ephy_tab_destroy_brsr_cb (EphyEmbed *embed, EphyTab *tab)
 {
+	GtkWidget *notebook;
+
 	g_return_if_fail (EPHY_IS_TAB (tab));
 	g_return_if_fail (tab->priv->window != NULL);
 
-	ephy_window_remove_tab (tab->priv->window, tab);
+	/* Do not use ephy_window_remove_tab because it will
+	   check for unsubmitted forms */
+	notebook = ephy_window_get_notebook (tab->priv->window);
+	ephy_notebook_remove_tab (EPHY_NOTEBOOK (notebook), tab);
 }
 
 static void
@@ -930,14 +969,12 @@ ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 {
 	GtkWidget *notebook;
 	EphyWindow *window;
-	GtkWidget *widget;
 
 	tab->priv->width = width;
 	tab->priv->height = height;
 
 	window = tab->priv->window;
 	notebook = ephy_window_get_notebook (window);
-	widget = GTK_WIDGET (embed);
 
 	/* Do not resize window with multiple tabs.
 	 * Do not resize window already showed because
@@ -947,14 +984,15 @@ ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 	    !tab->priv->visibility)
 	{
 		gtk_widget_set_size_request
-			(widget, width, height);
+			(GTK_WIDGET (tab), width, height);
 
 		/* HACK reset widget requisition after the container
 		 * has been resized. It appears to be the only way
 		 * to have the window sized according to embed
 		 * size correctly.
 		 */
-		g_idle_add (let_me_resize_hack, embed);
+
+		g_idle_add ((GSourceFunc) let_me_resize_hack, tab);
 	}
 }
 
@@ -1014,8 +1052,7 @@ clipboard_text_received_cb (GtkClipboard *clipboard,
 {
 	if (*weak_ptr != NULL && text != NULL)
 	{
-		EphyTab *tab = (EphyTab *) *weak_ptr;
-		EphyEmbed *embed = ephy_tab_get_embed (tab);
+		EphyEmbed *embed = (EphyEmbed *) *weak_ptr;
 
 		ephy_embed_load_url (embed, text);
 	}
@@ -1098,17 +1135,17 @@ ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 	{
 		/* See bug #133633 for why we do it this way */
 
-		/* We need to make sure we know if the tab is destroyed between
+		/* We need to make sure we know if the embed is destroyed between
 		 * requesting the clipboard contents, and receiving them.
 		 */
 		gpointer *weak_ptr;
 
 		weak_ptr = g_new (gpointer, 1);
-		*weak_ptr = tab;
-		g_object_add_weak_pointer (G_OBJECT (tab), weak_ptr);
+		*weak_ptr = embed;
+		g_object_add_weak_pointer (G_OBJECT (embed), weak_ptr);
 
 		gtk_clipboard_request_text
-			(gtk_widget_get_clipboard (GTK_WIDGET (tab->priv->embed),
+			(gtk_widget_get_clipboard (GTK_WIDGET (embed),
 						   GDK_SELECTION_PRIMARY),
 			 (GtkClipboardTextReceivedFunc) clipboard_text_received_cb,
 			 weak_ptr);
@@ -1132,7 +1169,7 @@ ephy_tab_security_change_cb (EphyEmbed *embed, EmbedSecurityLevel level,
 static void
 ephy_tab_init (EphyTab *tab)
 {
-	GObject *embed, *embed_widget;
+	GObject *embed;
 	EphyFaviconCache *cache;
 	char *id;
 
@@ -1158,12 +1195,11 @@ ephy_tab_init (EphyTab *tab)
 	tab->priv->setting_zoom = FALSE;
 	tab->priv->address_expire = TAB_ADDRESS_EXPIRE_NOW;
 
-	tab->priv->embed = EPHY_EMBED
-		(ephy_embed_factory_new_object ("EphyEmbed"));
-	g_assert (tab->priv->embed != NULL);
+	embed = ephy_embed_factory_new_object ("EphyEmbed");
+	g_assert (embed != NULL);
 
-	embed = G_OBJECT (tab->priv->embed);
-	embed_widget = G_OBJECT (tab->priv->embed);
+	gtk_container_add (GTK_CONTAINER (tab), GTK_WIDGET (embed));
+	gtk_widget_show (GTK_WIDGET (embed));
 
 	id = g_strdup_printf ("Tab%lu", tab_id++);
 
@@ -1177,15 +1213,6 @@ ephy_tab_init (EphyTab *tab)
 	g_signal_connect (tab->priv->action, "activate",
 			  G_CALLBACK (ephy_tab_action_activate_cb), tab);
 
-	/* set a pointer in the embed's widget back to the tab */
-	g_object_set_data (embed_widget, "ephy-tab", tab);
-
-	g_signal_connect_object (embed_widget, "parent_set",
-				 G_CALLBACK (ephy_tab_parent_set_cb),
-				 tab, 0);
-	g_signal_connect_object (embed_widget, "destroy",
-				 G_CALLBACK (ephy_tab_embed_destroy_cb),
-				 tab, 0);
 	g_signal_connect_object (embed, "link_message",
 				 G_CALLBACK (ephy_tab_link_message_cb),
 				 tab, 0);
@@ -1252,13 +1279,10 @@ ephy_tab_get_load_percent (EphyTab *tab)
 }
 
 static void
-ephy_tab_update_navigation_flags (EphyTab *tab)
+ephy_tab_update_navigation_flags (EphyTab *tab, EphyEmbed *embed)
 {
-	EphyEmbed *embed;
 	TabNavigationFlags flags = 0;
 
-	embed = tab->priv->embed;
-
 	if (ephy_embed_can_go_up (embed))
 	{
 		flags |= TAB_NAV_UP;
@@ -1312,7 +1336,7 @@ ephy_tab_get_status_message (EphyTab *tab)
 #define MAX_LABEL_LENGTH	32
 
 static void
-ephy_tab_set_title (EphyTab *tab, const char *new_title)
+ephy_tab_set_title (EphyTab *tab, EphyEmbed *embed, const char *new_title)
 {
 	char *title_short, *title_tmp;
 	char *title = NULL;
@@ -1326,7 +1350,7 @@ ephy_tab_set_title (EphyTab *tab, const char *new_title)
 		GnomeVFSURI *uri = NULL;
 		char *address;
 
-		address = ephy_embed_get_location (tab->priv->embed, TRUE);
+		address = ephy_embed_get_location (embed, TRUE);
 
 		if (address)
 		{
